<?php

namespace Drupal\vault\Plugin\VaultLeaseStorage;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Drupal\encrypt\EncryptServiceInterface;
use Drupal\encrypt\Exception\EncryptException;
use Drupal\encrypt\Exception\EncryptionMethodCanNotDecryptException;
use Drupal\vault\Plugin\VaultLeaseStorageBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a token-based authentication strategy for the vault client.
 *
 * @VaultLeaseStorage(
 *   id = "encrypted_state",
 *   label = "Encrypted State",
 *   description = @Translation("This authentication strategy encrypts secrets prior to storing in Drupal State"),
 *   provider = "encrypt"
 * )
 */
final class VaultLeaseEncryptedStateStorage extends VaultLeaseStorageBase implements PluginFormInterface, ConfigurableInterface {

  use StringTranslationTrait;

  /**
   * The storage handler.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface
   */
  protected $storage;

  /**
   * Vault client to obtain data from transit.
   *
   * @var \Drupal\vault\VaultClient
   */
  protected $client;

  /**
   * Entity Type Manager Service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityManager;

  /**
   * ID of EncryptionProfile to be used for encryption/decryption of data.
   *
   * @var string|null
   */
  protected $encryptionProfileId;

  /**
   * Encrypt module service.
   *
   * @var \Drupal\encrypt\EncryptServiceInterface
   */
  protected $encryptService;

  /**
   * VaultLeaseEncryptedStateStorage constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   String Translation service.
   * @param \Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface $storage
   *   Key/Value storage that supports expire times.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager service.
   * @param \Drupal\encrypt\EncryptServiceInterface $encrypt_service
   *   Encryption service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TranslationInterface $translation, KeyValueStoreExpirableInterface $storage, EntityTypeManagerInterface $entity_type_manager, EncryptServiceInterface $encrypt_service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->storage = $storage;
    $this->entityManager = $entity_type_manager;
    $this->setConfiguration($configuration);
    $this->encryptionProfileId = $configuration['encryption_profile'] ?? NULL;
    $this->encryptService = $encrypt_service;
    $this->setStringTranslation($translation);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation'),
      $container->get('keyvalue.expirable')->get('vault'),
      $container->get('entity_type.manager'),
      $container->get('encryption')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getLease($storage_key) {
    $item = $this->getLeaseRaw($storage_key);
    if (empty($item['data'])) {
      return NULL;
    }
    return $item['data'];
  }

  /**
   * Returns the raw state entry for a lease from Drupal's internal storage.
   *
   * @param string $storage_key
   *   The storage key. Something like "key:key_machine_id".
   *
   * @return mixed
   *   The response from state api.
   */
  protected function getLeaseRaw($storage_key) {
    $encrypted_data = $this->storage->get($storage_key);
    if ($encrypted_data == NULL) {
      return NULL;
    }

    if ($this->encryptionProfileId == NULL) {
      return NULL;
    }
    /** @var \Drupal\encrypt\EncryptionProfileInterface $encryption_profile */
    $encryption_profile = $this->entityManager
      ->getStorage('encryption_profile')
      ->load($this->encryptionProfileId);

    try {
      $serialized_data = $this->encryptService->decrypt($encrypted_data, $encryption_profile);
    }
    catch (EncryptException | EncryptionMethodCanNotDecryptException $e) {
      return NULL;
    }
    return json_decode($serialized_data, TRUE);

  }

  /**
   * {@inheritdoc}
   */
  public function getAllLeases() {
    $items = $this->storage->getAll();
    $returned = [];

    // If we don't have an encryption profile set return none.
    if ($this->encryptionProfileId == NULL) {
      return $returned;
    }

    foreach ($items as $key => $encrypted_data) {
      if ($encrypted_data == NULL) {
        $returned[$key] = NULL;
        continue;
      }

      /** @var \Drupal\encrypt\EncryptionProfileInterface $encryption_profile */
      $encryption_profile = $this->entityManager
        ->getStorage('encryption_profile')
        ->load($this->encryptionProfileId);

      try {
        $serialized_data = $this->encryptService->decrypt($encrypted_data, $encryption_profile);
      }
      catch (EncryptException | EncryptionMethodCanNotDecryptException $e) {
        $returned[$key] = NULL;
        continue;
      }
      $item = json_decode($serialized_data, TRUE);
      $returned[$key] = $item['data'];
    }
    return $returned;
  }

  /**
   * {@inheritdoc}
   */
  public function getLeaseId($storage_key) {
    $item = $this->getLeaseRaw($storage_key);
    if (empty($item)) {
      return FALSE;
    }
    return $item['lease_id'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteLease($storage_key): void {
    $this->storage->delete($storage_key);
  }

  /**
   * {@inheritdoc}
   */
  public function setLease($storage_key, $lease_id, $data, $expires): void {
    $payload = [
      'lease_id' => $lease_id,
      'data' => $data,
    ];
    $serialized_payload = json_encode($payload);
    if ($serialized_payload == FALSE) {
      throw new \Exception('failed to encode data in setLease');
    }

    if ($this->encryptionProfileId == NULL) {
      return;
    }

    /** @var \Drupal\encrypt\EncryptionProfileInterface $encryption_profile */
    $encryption_profile = $this->entityManager
      ->getStorage('encryption_profile')
      ->load($this->encryptionProfileId);

    try {
      $encrypted_data = $this->encryptService->encrypt($serialized_payload, $encryption_profile);
    }
    catch (EncryptException $e) {
      // The client will obtain a new lease as this one could not be stored.
      return;
    }
    $this->storage->setWithExpire($storage_key, $encrypted_data, $expires);
  }

  /**
   * {@inheritdoc}
   */
  public function updateLeaseExpires($storage_key, $new_expires): void {
    $data = $this->getLeaseRaw($storage_key);
    $this->setLease($storage_key, $data['lease_id'], $data['data'], $new_expires);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    // @todo Implement buildConfigurationForm() method.
    $available_profiles = [];
    $encryption_storage = $this->entityManager->getStorage('encryption_profile');
    /** @var \Drupal\encrypt\EncryptionProfileInterface $profile */
    foreach ($encryption_storage->loadMultiple() as $profile) {
      $available_profiles[$profile->id()] = $profile->label();
    }
    $form['encryption_profile'] = [
      '#type' => 'select',
      '#title' => $this->t('Encryption Profile'),
      '#options' => $available_profiles,
      '#default_value' => $this->encryptionProfileId,
      '#description' => $this->t(
        'The encryption profile that will be utilized for securing the data. If you do not see the profile you may <a href=":link">create a new profile</a>',
        [':link' => Url::fromRoute('entity.encryption_profile.add_form')->toString()]
      ),

    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    // No additional validation required.
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration(): array {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration): void {
    $this->configuration = $configuration;
    $this->encryptionProfileId = $configuration['encryption_profile'] ?? NULL;
  }

  /**
   * Gets default configuration for this plugin.
   *
   * @return array
   *   An associative array with the default configuration.
   */
  public function defaultConfiguration(): array {
    return [
      'encryption_profile' => NULL,
    ];
  }

}
