<?php

namespace Drupal\vault\Plugin\VaultLeaseStorage;

use Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface;
use Drupal\vault\Plugin\VaultLeaseStorageBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a token-based authentication strategy for the vault client.
 *
 * @VaultLeaseStorage(
 *   id = "state",
 *   label = "Drupal State",
 *   description = @Translation("This authentication strategy uses Drupal State to store leases.."),
 * )
 */
final class VaultLeaseStateStorage extends VaultLeaseStorageBase {

  /**
   * The storage handler.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface
   */
  protected $storage;

  /**
   * VaultLeaseStateStorage constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface $storage
   *   Key/Value storage that supports expire times.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, KeyValueStoreExpirableInterface $storage) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->storage = $storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('keyvalue.expirable')->get('vault_lease')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getLease($storage_key) {
    $item = $this->getLeaseRaw($storage_key);
    if (empty($item['data'])) {
      return NULL;
    }
    return $item['data'];
  }

  /**
   * Returns the raw state entry for a lease from Drupal's internal storage.
   *
   * @param string $storage_key
   *   The storage key. Something like "key:key_machine_id".
   *
   * @return mixed
   *   The response from state api.
   */
  protected function getLeaseRaw($storage_key) {
    return $this->storage->get($storage_key);
  }

  /**
   * {@inheritdoc}
   */
  public function getAllLeases() {
    $items = $this->storage->getAll();
    $returned = [];
    foreach ($items as $key => $item) {
      $returned[$key] = $item['data'];
    }
    return $returned;
  }

  /**
   * {@inheritdoc}
   */
  public function getLeaseId($storage_key) {
    $item = $this->getLeaseRaw($storage_key);
    if (empty($item)) {
      return FALSE;
    }
    return $item['lease_id'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteLease($storage_key): void {
    $this->storage->delete($storage_key);
  }

  /**
   * {@inheritdoc}
   */
  public function setLease($storage_key, $lease_id, $data, $expires): void {
    $payload = [
      'lease_id' => $lease_id,
      'data' => $data,
    ];
    $this->storage->setWithExpire($storage_key, $payload, $expires);
  }

  /**
   * {@inheritdoc}
   */
  public function updateLeaseExpires($storage_key, $new_expires): void {
    $data = $this->storage->get($storage_key);
    $this->setLease($storage_key, $data['lease_id'], $data['data'], $new_expires);
  }

}
