<?php

namespace Drupal\vault;

use Drupal\vault\Exceptions\ClientException;
use Drupal\vault\Plugin\VaultLeaseStorageInterface;
use Vault\CachedClient;

/**
 * Wrapper for \Vault\Client providing some helper methods.
 *
 * It also acts as a translation layer between vault leases and drupal entities
 * implementing leases (like keys).
 */
final class VaultClient extends CachedClient implements VaultClientInterface {

  public const API = 'v1';

  /**
   * The lease storage object.
   *
   * @var \Drupal\vault\Plugin\VaultLeaseStorageInterface
   */
  protected $leaseStorage;

  /**
   * {@inheritdoc}
   */
  public function setLeaseStorage(VaultLeaseStorageInterface $leaseStorage): void {
    $this->leaseStorage = $leaseStorage;
  }

  /**
   * {@inheritdoc}
   */
  public function listMounts() {
    return $this->read('/sys/mounts')->getData();
  }

  /**
   * {@inheritdoc}
   */
  public function listSecretEngineMounts(array $engine_types) {
    $data = $this->listMounts();
    if ($data == NULL) {
      return [];
    }
    return array_filter($data, function ($v) use ($engine_types) {
      return in_array($v['type'], $engine_types);
    });
  }

  /**
   * {@inheritdoc}
   */
  public function storeLease($storage_key, $lease_id, $data, $expires): void {
    $this->leaseStorage->setLease($storage_key, $lease_id, $data, $expires);
  }

  /**
   * {@inheritdoc}
   */
  public function retrieveLease($storage_key) {
    return $this->leaseStorage->getLease($storage_key);
  }

  /**
   * {@inheritdoc}
   */
  public function revokeLease($storage_key) {
    $this->logger->debug(sprintf("attempting to revoke lease for %s", $storage_key));
    $lease_id = $this->leaseStorage->getLeaseId($storage_key);
    if (empty($lease_id)) {
      $this->logger->error(sprintf("could not find lease for %s", $storage_key));
      return TRUE;
    }

    try {
      $data["lease_id"] = $lease_id;
      $encoded_data = json_encode($data);
      if ($encoded_data === FALSE) {
        throw new ClientException("Failed to encode request to revoke lease for " . $storage_key);
      }
      $this->put($this->buildPath("/sys/leases/revoke"), $encoded_data);
    }
    catch (\Exception $e) {
      $this->logger->error(sprintf("Failed revoking lease %s", $lease_id));
      $this->leaseStorage->deleteLease($lease_id);
      return FALSE;
    }

    $this->leaseStorage->deleteLease($lease_id);
    return TRUE;

  }

  /**
   * {@inheritdoc}
   */
  public function renewLease($storage_key, $increment) {
    $this->logger->debug(sprintf("attempting to renew lease for %s", $storage_key));
    $lease_id = $this->leaseStorage->getLeaseId($storage_key);
    try {
      if (empty($lease_id)) {
        throw new ClientException("no valid lease for " . $storage_key);
      }

      $data = [
        "lease_id" => $lease_id,
        "increment" => $increment,
      ];
      $encoded_data = json_encode($data);
      if ($encoded_data === FALSE) {
        throw new ClientException("Failed to encode request to renew lease for " . $storage_key);
      }
      $response = $this->put($this->buildPath("/sys/leases/renew"), $encoded_data);
      if (is_null($response->getRequestId())) {
        throw new ClientException("null response from server renewing lease for " . $storage_key);
      }

      $new_expires = $response->getLeaseDuration();
      if ($new_expires === NULL) {
        throw new ClientException("null duration from server renewing lease for " . $storage_key);
      }
      $this->leaseStorage->updateLeaseExpires($storage_key, $new_expires);

    }
    catch (\Exception $e) {
      $this->logger->error(sprintf("Failed renewing lease %s", $lease_id));
      return FALSE;
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function renewAllLeases($increment): void {
    $leases = $this->leaseStorage->getAllLeases();
    foreach ($leases as $key => $value) {
      $response = $this->renewLease($key, $increment);
      if (!$response) {
        // Failed to renew the lease - remove it from state.
        $this->logger->info(sprintf("revoking expired lease for %s", $key));
        $this->revokeLease($key);
      }
    }
  }

}
