<?php

namespace Drupal\vault;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\vault\Plugin\VaultAuthManagerInterface;
use Drupal\vault\Plugin\VaultLeaseStorageManagerInterface;
use GuzzleHttp\Psr7\Uri;
use Http\Client\HttpClient;
use Psr\Http\Message\RequestFactoryInterface;
use Psr\Http\Message\StreamFactoryInterface;
use Psr\Log\LoggerInterface;
use Vault\Exceptions\AuthenticationException;
use Cache\Adapter\PHPArray\ArrayCachePool;

/**
 * Factory class for Vault client.
 *
 * @package Drupal\vault
 */
final class VaultClientFactory {

  /**
   * Creates an Vault Client instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Psr\Log\LoggerInterface $logger
   *   Logger service.
   * @param \Drupal\vault\Plugin\VaultAuthManagerInterface $vault_auth_manager
   *   Vault Auth Plugin Manager service.
   * @param \Drupal\vault\Plugin\VaultLeaseStorageManagerInterface $vault_lease_storage_manager
   *   Vault Lease Storage Plugin Manager service.
   * @param \Http\Client\HttpClient $http_client
   *   PSR 17 http client.
   * @param \Psr\Http\Message\RequestFactoryInterface $request_factory
   *   PSR 17 request factory.
   * @param \Psr\Http\Message\StreamFactoryInterface $stream_factory
   *   Stream Factory.
   * @param bool $set_lease_storage
   *   True if factory should set lease storage. False if caller will be
   *   responsible to set storage or avoid storage.
   *
   * @return \Drupal\vault\VaultClientInterface
   *   The client.
   *
   * @throws \Vault\Exceptions\AuthenticationException
   *   Thrown when the client is unable to authenticate.
   */
  public static function createInstance(ConfigFactoryInterface $configFactory, LoggerInterface $logger, VaultAuthManagerInterface $vault_auth_manager, VaultLeaseStorageManagerInterface $vault_lease_storage_manager, HttpClient $http_client, RequestFactoryInterface $request_factory, StreamFactoryInterface $stream_factory, bool $set_lease_storage) {
    $settings = $configFactory->get('vault.settings');

    $base_uri = new Uri($settings->get('base_url'));
    $client = new VaultClient($base_uri, $http_client, $request_factory, $stream_factory, $logger);
    $client->enableReadCache();
    $client->setCache(new ArrayCachePool());

    // Allow some clients to be created without a lease storage.
    // Care must be taken to either set a lease storage by the caller
    // or to never call the lease storage.
    if ($set_lease_storage) {
      $lease_plugin_name = $settings->get('plugin_lease_storage') ?: 'state';
      $lease_plugin_config = $settings->get('lease_storage_plugin_config');
      if (!is_array($lease_plugin_config)) {
        $lease_plugin_config = [];
      }
      /** @var \Drupal\vault\Plugin\VaultLeaseStorageInterface $lease_plugin */
      $lease_plugin = $vault_lease_storage_manager->createInstance($lease_plugin_name, $lease_plugin_config);
      $client->setLeaseStorage($lease_plugin);
    }

    // Load up auth strategy.
    try {
      $auth_plugin_name = $settings->get('plugin_auth');
      if (empty($auth_plugin_name)) {
        throw new AuthenticationException("No auth plugin configured");
      }

      $auth_plugin_config = $settings->get('auth_plugin_config');
      if (!is_array($auth_plugin_config)) {
        $auth_plugin_config = [];
      }
      /** @var \Drupal\vault\Plugin\VaultAuthInterface $plugin */
      $plugin = $vault_auth_manager->createInstance($auth_plugin_name, $auth_plugin_config);
      $auth_strategy = $plugin->getAuthenticationStrategy();

      $authenticated = $client->setAuthenticationStrategy($auth_strategy)->authenticate();
      if (!$authenticated) {
        throw new AuthenticationException("Failed to authenticate");
      }
    }
    catch (\Exception $e) {
      $logger->error(sprintf("[%s] %s", get_class($e), $e->getMessage()));
      throw $e;
    }

    return $client;
  }

}
